/***************************************************************************
 *                                                                         *
 *   MAILCRAWL                                                             *
 *   =========                                                             *
 *                                                                         *
 *   Copyright (C)                                                         *
 *   Juan Pedro Bolivar Puente                                             *
 *   Francisco Herrero Pérez                                               *
 *   2007                                                                  *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <string.h>

#include "cache.h"

#include "gl_oset.h"
#include "gl_rbtree_oset.h"

static idxstr_p
idxstr_create(char* string, int index)
{
  idxstr_p e;
  
  e = malloc(sizeof(idxstr_t));

  if (e)
    {
      e->str = malloc (strlen (string) + 1);
      strcpy (e->str, string);
      e->idx = index;
    }
  
  return e;
}

INLINE void
idxstr_free(idxstr_p e)
{
  free(e->str);
  free(e);
}

INLINE int
idxstr_cmp(idxstr_p a, idxstr_p b)
{
  return strcmp(a->str, b->str);
}

cache_p
cache_create()
{
  cache_p c;

  c = malloc(sizeof(cache_t));

  if (c)
    {
      c->index = 0;
      c->data = gl_oset_create_empty(GL_RBTREE_OSET,
				     (gl_setelement_compar_fn) &idxstr_cmp,
				     (gl_setelement_dispose_fn) &idxstr_free);
    }

  return c;
}

void
cache_free(cache_p c)
{
  gl_oset_free(c->data);
  free(c);
}

void
cache_adjust(cache_p c, int max_size)
{
  gl_oset_iterator_t it;
  idxstr_p e;
  size_t min_index;

  if (max_size < c->index)
    {
      min_index = c->index - (c->index - max_size);

      it = gl_oset_iterator(c->data);
      while(gl_oset_iterator_next(&it, (void*)&e))
	{
	  if (e->idx < min_index)
	    gl_oset_remove(c->data, (void*)e);
	  else
	    e->idx -= min_index;
	}

      c->index -= min_index;
    }
}

bool
cache_add(cache_p c, char* str)
{
  idxstr_p elem = idxstr_create(str, c->index);
  
  if (gl_oset_add(c->data, elem))
    {
      c->index++;
      return true;
    }
  else
    {
      idxstr_free(elem);
      return false;
    }  
}
