/***************************************************************************
 *                                                                         *
 *   MAILCRAWL                                                             *
 *   =========                                                             *
 *                                                                         *
 *   Copyright (C)                                                         *
 *   Juan Pedro Bolivar Puente                                             *
 *   Francisco Manuel Herrero Pérez                                        *
 *   2007                                                                  *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdarg.h>

#include "log.h"

static FILE* _output = NULL;
static log_level_t _level = LOG_INFO;


void
log_set_file(FILE* log)
{
  _output = log;
}

void log_set_level(log_level_t lev)
{
  _level = lev;
}

static const char*
log_level_string(log_level_t lev)
{
  switch(lev)
    {
    case LOG_FATAL:
      return "fatal: ";
    case LOG_ERROR:
      return "error: ";
    case LOG_WARNING:
      return "warning: ";
    default:
      return "";
    }
}

void
log_msg(log_level_t sev, const char* msg, ...)
{
  va_list ap;
  FILE* stdfile;
  
  if (_level <= sev)
    {
      stdfile = sev <= LOG_INFO ? stdout : stderr;
      
      fputs(log_level_string(sev), stdfile);
      va_start(ap, msg);
      vfprintf(stdfile, msg, ap);
      va_end(ap);
      fputs("\n", stdfile);
      
      if (_output != NULL)
	{
	  fputs(log_level_string(sev), _output);
	  va_start(ap, msg);
	  vfprintf(_output, msg, ap);
	  va_end(ap);
	  fputs("\n", _output);
	}
    }
}

