/***************************************************************************
 *                                                                         *
 *   MAILCRAWL                                                             *
 *   =========                                                             *
 *                                                                         *
 *   Copyright (C)                                                         *
 *   Juan Pedro Bolivar Puente                                             *
 *   Francisco Manuel Herrero Pérez                                        *
 *   2007                                                                  *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

%{

#include "mailcrawl.h"
#include "log.h"

typedef struct extra_data_s
{
  mailcrawl_p crawler;
  const char* url_base;
} extra_data_t;

#define YY_EXTRA_TYPE extra_data_t*

INLINE char*
take_quotes(char* str)
{
  str[strlen(str)-1] = '\0';
  return str+1;
}

%}

%option reentrant

ext		([xX]?[hH][tT][mM][lL]?|[xX][mM][lL]|[pP][hH][pP][1-5]?|[aA][sS][pP][xX]|[tT][xX][tT])
proto           ([hH][tT][tT][pP][Ss]?|[fF][tT][pP])
subdomain       ([wW]{3})[[:alnum:]]?
basedomain      [[:alpha:]]{2}|([cC][oO][mM]|[oO][rR][gG]|[nN][eE][tT]|[gG][oO][vV]|[bB][iI][zZ]|[iI][nN][fF][oO]|[nN][aA][mM][eE]|[aA][eE][rR][oO]|[bB][iI][zZ]|[iI][nN][fF][oO]|[jJ][oO][bB][sS]|[mM][uU][sS][eE][uU][mM])

dir_char        [[:alnum:]\_\-\b\~\=\+\&\*\%]
file_char	{dir_char}|\.
domain_char	[[:alnum:]\-\.]
user_char	[[:alnum:]\_\-\.]

ip_add          ([:digit:]{1,3}){4}
domain          {domain_char}+\.{basedomain}
host            ({ip_add}|{domain})
user            {user_char}+(":"{user_char}+)?@
url_base        ({proto}"://"{user}?|{user}?{subdomain}\.){host}
params          "?"{file_char}*
file            {file_char}*"."{ext}
directory	{dir_char}+
path            ({file_char}|"/")*"/"

email		{user_char}+@{host}

url_dir	        {url_base}({path}{directory}?{params}?)?
url_file        {url_base}{path}{file}{params}?

rel_dir		{path}{directory}{params}?
rel_file	{path}?{file}{params}?

%%

{email}			mailcrawl_put_email(yyextra->crawler, yytext);
\"{url_dir}\"		mailcrawl_add_url(yyextra->crawler, take_quotes(yytext), URL_DIR);
\"{url_file}\"		mailcrawl_add_url(yyextra->crawler, take_quotes(yytext), URL_FILE);
\"{rel_dir}\"		mailcrawl_add_rel(yyextra->crawler, yyextra->url_base, take_quotes(yytext), URL_DIR);
\"{rel_file}\"		mailcrawl_add_rel(yyextra->crawler, yyextra->url_base, take_quotes(yytext), URL_FILE);

\n|.			{}

%%

bool
mailcrawl_lex(mailcrawl_p mc, const char* file, const char* url_base)
{
  FILE* fin;
  yyscan_t scanner;

  log_msg(LOG_INFO, "Analizando...");

  fin = fopen(file, "r");
  if (!fin)
    {
      log_msg(LOG_ERROR,
	      "No se pudo abrir para lectura el fichero temporal %s.",
	      file);
      return ERROR;
    }

  extra_data_t* extra = malloc (sizeof (extra_data_t));
  extra->crawler = mc;
  extra->url_base = url_base;
  
  yylex_init (&scanner);
  
  yyset_in (fin, scanner);
  yyset_extra (extra, scanner);

  yylex (scanner);

  yylex_destroy (scanner);
  free (extra);
  
  return OK;
}
