/***************************************************************************
 *                                                                         *
 *   MAILCRAWL                                                             *
 *   =========                                                             *
 *                                                                         *
 *   Copyright (C)                                                         *
 *   Juan Pedro Bolivar Puente                                             *
 *   Francisco Manuel Herrero Pérez                                        *
 *   2007                                                                  *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#ifndef MAILCRAWL_H
#define MAILCRAWL_H

#include <curl/curl.h>

#include "cache.h"
#include "queue.h"
#include "thread-pool.h"

enum url_types
  {
    URL_DIR,
    URL_FILE,
  };

struct mailcrawl_s
{
  FILE* output;
  int max_cache;
  int max_pages;
  queue_p next_pages;
  cache_p found_pages;
  cache_p found_mails;
  bool decreasing;
  
  thread_pool_p thread_pool;
  pthread_mutex_t mutex;
};

typedef struct mailcrawl_s mailcrawl_t;
typedef mailcrawl_t* mailcrawl_p;

void
mailcrawl_kill_handler();

mailcrawl_p
mailcrawl_create();

bool
mailcrawl_next(mailcrawl_p mc, char** url_base, char** url);

bool
mailcrawl_fetch(mailcrawl_p mc, const char* file, const char* url);

bool
mailcrawl_lex(mailcrawl_p mc, const char* file, const char* url_base);

void
mailcrawl_run(mailcrawl_p mc, size_t num_threads);

void
mailcrawl_free(mailcrawl_p mc);

bool
mailcrawl_put_email(mailcrawl_p mc, const char* email);

bool
mailcrawl_add_url(mailcrawl_p mc, const char* url, int type);

void
mailcrawl_load_urls(mailcrawl_p mc, const char** urls, int n_urls);

bool
mailcrawl_add_rel(mailcrawl_p mc,
		  const char* url_base,
		  const char* file,
		  int type);

bool
mailcrawl_load_resume_file(mailcrawl_p mc, const char* file);

bool
mailcrawl_save_resume_file(mailcrawl_p mc, const char* file);

INLINE bool
mailcrawl_empty(mailcrawl_p mc)
{
  return queue_empty(mc->next_pages);
}

INLINE void
mailcrawl_set_output(mailcrawl_p mc, FILE* out)
{
  mc->output = out;
}

INLINE FILE*
mailcrawl_get_output(mailcrawl_p mc)
{
  return mc->output;
}

INLINE void
mailcrawl_set_max_pages(mailcrawl_p mc, int max)
{
  mc->max_pages = max;
}

INLINE void
mailcrawl_set_max_cache(mailcrawl_p mc, int max)
{
  mc->max_cache = max;
}

#endif /* MAILCRAWL_H */
