/***************************************************************************
 *                                                                         *
 *   MAILCRAWL                                                             *
 *   =========                                                             *
 *                                                                         *
 *   Copyright (C)                                                         *
 *   Juan Pedro Bolivar Puente                                             *
 *   Francisco Manuel Herrero Pérez                                        *
 *   2007                                                                  *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#include "queue.h"

queue_p
queue_create(queue_dispose_fn dispose_fn)
{
  queue_p q = malloc(sizeof(queue_t));

  if (q)
    {
      q->first = NULL;
      q->last = NULL;
      q->size = 0;
      q->dispose_fn = dispose_fn;
    }

  return q;
}

int
queue_push(queue_p q, void* data)
{
  qnode_p n = malloc(sizeof(qnode_t));

  if (n == NULL)
    return ERROR;

  n->next = NULL;
  n->data = data;
  
  if (q->last)
    q->last->next = n;
  q->last = n;

  if (!q->first)
    q->first = n;

  q->size++;
  
  return OK;
}

INLINE qnode_p
qnode_free(queue_p q, qnode_p r)
{
  qnode_p n = r->next;

  if (q->dispose_fn)
    q->dispose_fn(r->data);
  free(r);
  
  return n;
}

void
queue_pop(queue_p q)
{
  if (q->first)
    q->first = qnode_free(q, q->first);

  if (!q->first)
    {
      q->last = NULL;
    }

  q->size--;
}

void
queue_free(queue_p q)
{
  if (q->first)
    while ((q->first = qnode_free(q, q->first)));
  free(q);
}
