/***************************************************************************
 *                                                                         *
 *   MAILCRAWL                                                             *
 *   =========                                                             *
 *                                                                         *
 *   Copyright (C)                                                         *
 *   Juan Pedro Bolivar Puente                                             *
 *   Francisco Manuel Herrero Pérez                                        *
 *   2007                                                                  *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#ifndef QUEUE_H
#define QUEUE_H

#include <stdlib.h>
#include "common.h"

typedef void(*queue_dispose_fn)(void*);

struct qnode_s
{
  void* data;
  struct qnode_s* next;
};

typedef struct qnode_s qnode_t;
typedef qnode_t* qnode_p;

struct queue_s
{
  qnode_p first;
  qnode_p last;
  size_t size;
  queue_dispose_fn dispose_fn;
};

typedef struct queue_s queue_t;
typedef queue_t* queue_p;

queue_p
queue_create(queue_dispose_fn dispose_fn);

int
queue_push(queue_p q, void* data);

void
queue_pop(queue_p q);

void
queue_free(queue_p q);

INLINE void*
queue_front(queue_p q)
{
  return q->first->data;
}

INLINE bool
queue_empty(queue_p q)
{
  return (q->first == NULL);
}

INLINE size_t
queue_size(queue_p q)
{
  return q->size;
}

#endif /* QUEUE_H */
